from typing import Union
import pathlib

from qtpy import QtQuick, QtCore

from prettyqt import gui, core
from prettyqt.utils import bidict


RESIZE_MODES = bidict(
    view_to_root_object=QtQuick.QQuickView.SizeViewToRootObject,
    root_object_to_view=QtQuick.QQuickView.SizeRootObjectToView,
)

STATUS = bidict(
    null=QtQuick.QQuickView.Null,
    ready=QtQuick.QQuickView.Ready,
    loading=QtQuick.QQuickView.Loading,
    error=QtQuick.QQuickView.Error,
)


QtQuick.QQuickView.__bases__ = (gui.Window,)


class QuickView(QtQuick.QQuickView):
    def set_source(self, source: Union[str, pathlib.Path, QtCore.QUrl]):
        if isinstance(source, pathlib.Path):
            source = str(source)
        if isinstance(source, str):
            source = core.Url.fromLocalFile(source)
        self.setSource(source)

    def get_source(self) -> pathlib.Path:
        return pathlib.Path(self.source().toLocalFile())

    def get_status(self) -> str:
        return STATUS.inv[self.status()]



from typing import Dict

from qtpy import QtGui

from prettyqt import gui, core
from prettyqt.utils import colors, bidict


ROLES = bidict(
    background=QtGui.QPalette.Background,  # same as Window
    foreground=QtGui.QPalette.Foreground,  # same as WindowText
    base=QtGui.QPalette.Base,
    alternate_base=QtGui.QPalette.AlternateBase,
    tool_tip_base=QtGui.QPalette.ToolTipBase,
    tool_tip_text=QtGui.QPalette.ToolTipText,
    placeholder_text=QtGui.QPalette.PlaceholderText,
    text=QtGui.QPalette.Text,
    button=QtGui.QPalette.Button,
    button_text=QtGui.QPalette.ButtonText,
    bright_text=QtGui.QPalette.BrightText,
)

GROUPS = bidict(
    disabled=QtGui.QPalette.Disabled,
    active=QtGui.QPalette.Active,
    inactive=QtGui.QPalette.Inactive,
)


class Palette(QtGui.QPalette):
    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def __getitem__(self, index: str) -> gui.Color:
        return self.get_color(index)

    def __setitem__(self, index: str, value: colors.ColorType):
        self.set_color(index, value)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def highlight_inactive(self):
        color = self.color(self.Active, self.Highlight)
        self.setColor(self.Inactive, self.Highlight, color)

    def set_color(self, role: str, color: colors.ColorType, group: str = "active"):
        color = colors.get_color(color)
        self.setColor(GROUPS[group], ROLES[role], color)

    def get_colors(self, group: str = "active") -> Dict[str, gui.Color]:
        return {k: self.get_color(k, group) for k in ROLES.keys()}

    def get_color(self, role: str, group: str = "active") -> gui.Color:
        return gui.Color(self.color(GROUPS[group], ROLES[role]))


if __name__ == "__main__":
    pal = Palette()
    print(pal.get_colors())



# -*- coding: utf-8 -*-

from qtpy import QtCore


class Size(QtCore.QSize):
    def __repr__(self):
        return f"Size({self.width()}, {self.height()})"

    def __getitem__(self, index) -> int:
        return (self.width(), self.height())[index]

    def __reduce__(self):
        return (self.__class__, (self.width(), self.height()))


if __name__ == "__main__":
    size = Size(10, 20)
    print(tuple(size))



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtNetwork

from prettyqt import core, network


QtNetwork.QTcpServer.__bases__ = (core.Object,)


class TcpServer(QtNetwork.QTcpServer):
    def listen(self, address: Union[str, QtNetwork.QHostAddress], port: int = 0) -> bool:
        if isinstance(address, str):
            address = network.HostAddress(address)
        return super().listen(address, port)

    def get_server_address(self) -> network.HostAddress:
        return network.HostAddress(self.serverAddress())

    def get_proxy(self) -> network.NetworkProxy:
        return network.NetworkProxy(self.proxy())

    def get_server_error(self) -> str:
        return network.abstractsocket.SOCKET_ERROR.inv[self.serverError()]



# -*- coding: utf-8 -*-

from typing import List, Union

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets


class WebEngineScriptCollection:
    def __init__(self, item: QtWebEngineWidgets.QWebEngineScriptCollection):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def __getitem__(self, index: str) -> QtWebEngineWidgets.QWebEngineScript:
        return self.item.findScript(index)

    def __len__(self):
        return self.item.count()

    def __iter__(self):
        return iter(self.item.toList())

    def __contains__(self, other: QtWebEngineWidgets.QWebEngineScript):
        return self.item.contains(other)

    def __add__(
        self,
        other: Union[
            QtWebEngineWidgets.QWebEngineScript, List[QtWebEngineWidgets.QWebEngineScript]
        ],
    ):
        self.item.insert(other)
        return self


if __name__ == "__main__":
    from prettyqt import widgets, webenginewidgets

    app = widgets.app()
    page = webenginewidgets.WebEnginePage()
    scripts = page.scripts()
    script = webenginewidgets.WebEngineScript()
    script.setName("test")
    item = WebEngineScriptCollection(scripts)
    assert bool(item) is False
    item += script
    assert script in item
    assert len(item) == 1
    assert bool(item) is True
    app.main_loop()



from qtpy import QtQuick

from prettyqt import gui, core, quick
from prettyqt.utils import bidict, InvalidParamError


PERFORMANCE_HINT = bidict(
    fast_fbo_resizing=QtQuick.QQuickPaintedItem.FastFBOResizing,
)

RENDER_TARGET = bidict(
    image=QtQuick.QQuickPaintedItem.Image,
    framebuffer_object=QtQuick.QQuickPaintedItem.FramebufferObject,
    inverted_y_framebuffer_object=QtQuick.QQuickPaintedItem.InvertedYFramebufferObject,
)


QtQuick.QQuickPaintedItem.__bases__ = (quick.QuickItem,)


class QuickPaintedItem(QtQuick.QQuickPaintedItem):
    def get_fillcolor(self) -> gui.Color:
        return gui.Color(self.fillColor())

    def get_texture_size(self) -> core.Size:
        return core.Size(self.textureSize())

    def set_render_target(self, target: str):
        """Set the render target.

        Allowed values are "image", "framebuffer_object", "inverted_y_framebuffer_object"

        Args:
            target: render target to use

        Raises:
            InvalidParamError: render target does not exist
        """
        if target not in RENDER_TARGET:
            raise InvalidParamError(target, RENDER_TARGET)
        self.setRenderTarget(RENDER_TARGET[target])

    def get_render_target(self) -> str:
        """Return the render target.

        Possible values: "image", "framebuffer_object", "inverted_y_framebuffer_object"

        Returns:
            render target
        """
        return RENDER_TARGET.inv[self.renderTarget()]



# -*- coding: utf-8 -*-

from typing import List
import contextlib

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import bidict, InvalidParamError

CURSOR_SHAPES = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
    size_vertical=QtCore.Qt.SizeVerCursor,
    size_horizonal=QtCore.Qt.SizeHorCursor,
    size_topright=QtCore.Qt.SizeBDiagCursor,
    size_topleft=QtCore.Qt.SizeFDiagCursor,
    size_all=QtCore.Qt.SizeAllCursor,
    blank=QtCore.Qt.BlankCursor,
    split_vertical=QtCore.Qt.SplitVCursor,
    split_horizontal=QtCore.Qt.SplitHCursor,
    pointing_hand=QtCore.Qt.PointingHandCursor,
    forbidden=QtCore.Qt.ForbiddenCursor,
    open_hand=QtCore.Qt.OpenHandCursor,
    closed_hand=QtCore.Qt.ClosedHandCursor,
    whats_this=QtCore.Qt.WhatsThisCursor,
    busy=QtCore.Qt.BusyCursor,
    drag_move=QtCore.Qt.DragMoveCursor,
    drag_copy=QtCore.Qt.DragCopyCursor,
    drag_link=QtCore.Qt.DragLinkCursor,
    bitmap=QtCore.Qt.BitmapCursor,
)

LAYOUT_DIRECTIONS = bidict(
    left_to_right=QtCore.Qt.LeftToRight,
    right_to_left=QtCore.Qt.RightToLeft,
    auto=QtCore.Qt.LayoutDirectionAuto,
)

QtGui.QGuiApplication.__bases__ = (core.CoreApplication,)


class GuiApplication(QtGui.QGuiApplication):
    @classmethod
    @contextlib.contextmanager
    def override_cursor(cls, cursor: str):
        cls.set_override_cursor(cursor)
        yield cursor
        cls.restore_override_cursor()

    @classmethod
    def set_override_cursor(cls, cursor: str):
        cursor = gui.Cursor(CURSOR_SHAPES[cursor])
        cls.setOverrideCursor(cursor)

    @classmethod
    def restore_override_cursor(cls):
        cls.restoreOverrideCursor()

    @classmethod
    def get_clipboard(cls) -> gui.Clipboard:
        return gui.Clipboard(cls.clipboard())

    def set_layout_direction(self, direction: str):
        """Set layout direction.

        Valid values: "left_to_right", "right_to_left", "auto"

        Args:
            direction: layout direction

        Raises:
            InvalidParamError: layout direction does not exist
        """
        if direction not in LAYOUT_DIRECTIONS:
            raise InvalidParamError(direction, LAYOUT_DIRECTIONS)
        self.setLayoutDirection(LAYOUT_DIRECTIONS[direction])

    def get_layout_direction(self) -> str:
        """Get the current layout direction.

        Possible values: "left_to_right", "right_to_left", "auto"

        Returns:
            layout direction
        """
        return LAYOUT_DIRECTIONS.inv[self.layoutDirection()]

    def get_primary_screen(self) -> gui.Screen:
        return gui.Screen(self.primaryScreen())

    def get_screen_at(self, point: QtCore.QPoint) -> gui.Screen:
        return gui.Screen(self.screenAt(point))

    def get_screens(self) -> List[gui.Screen]:
        return [gui.Screen(i) for i in self.screens()]

    def get_input_method(self) -> gui.InputMethod:
        return gui.InputMethod(self.inputMethod())



# -*- coding: utf-8 -*-

from typing import Union, Optional
import pathlib

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWinExtras  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWinExtras

from prettyqt import gui
from prettyqt.utils import bidict


TYPES = bidict(
    custom=QtWinExtras.QWinJumpListCategory.Custom,
    recent=QtWinExtras.QWinJumpListCategory.Recent,
    frequent=QtWinExtras.QWinJumpListCategory.Frequent,
    tasks=QtWinExtras.QWinJumpListCategory.Tasks,
)


class WinJumpListCategory(QtWinExtras.QWinJumpListCategory):
    def __contains__(self, item):
        return item in self.items()

    def __len__(self) -> int:
        return self.count()

    def __bool__(self):
        return not self.isEmpty()

    def get_type(self) -> str:
        return TYPES.inv[self.type()]

    def add_destination(self, destination: Union[str, pathlib.Path]) -> None:
        self.addDestination(str(destination))

    def set_title(self, title: str) -> None:
        self.setTitle(title)

    def add_link(
        self,
        title: str,
        exe_path: Union[str, pathlib.Path],
        arguments: Optional[list] = None,
        icon: gui.icon.IconType = None,
    ) -> None:
        icon = gui.icon.get_icon(icon)
        if arguments is None:
            arguments = []
        self.addLink(icon, title, str(exe_path), arguments)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    cat = WinJumpListCategory()
    len(cat)
    app.main_loop()



# -*- coding: utf-8 -*-
from qtpy import QtCore

from prettyqt import core


QtCore.QStringListModel.__bases__ = (core.AbstractListModel,)


class StringListModel(QtCore.QStringListModel):
    def serialize_fields(self):
        return dict(
            string_list=self.stringList(),
        )



from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionTitleBar.__bases__ = (widgets.StyleOptionComplex,)


class StyleOptionTitleBar(QtWidgets.QStyleOptionTitleBar):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionGraphicsItem.__bases__ = (widgets.StyleOption,)


class StyleOptionGraphicsItem(QtWidgets.QStyleOptionGraphicsItem):
    pass



# -*- coding: utf-8 -*-

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from prettyqt import positioning

QtPositioning.QGeoCircle.__bases__ = (positioning.GeoShape,)


class GeoCircle(QtPositioning.QGeoCircle):
    def __repr__(self):
        return f"GeoCircle({self.get_center()!r}, {self.radius()})"

    def get_center(self) -> positioning.GeoCoordinate:
        return positioning.GeoCoordinate(self.center())


if __name__ == "__main__":
    coord = positioning.GeoCoordinate(1, 1)
    circle = GeoCircle(coord, 0.5)
    print(str(circle))
    print(repr(circle))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prettyqt',
 'prettyqt.__pyinstaller',
 'prettyqt.bluetooth',
 'prettyqt.charts',
 'prettyqt.constants',
 'prettyqt.core',
 'prettyqt.custom_delegates',
 'prettyqt.custom_models',
 'prettyqt.custom_validators',
 'prettyqt.custom_widgets',
 'prettyqt.custom_widgets.regexeditor',
 'prettyqt.gui',
 'prettyqt.location',
 'prettyqt.multimedia',
 'prettyqt.multimediawidgets',
 'prettyqt.network',
 'prettyqt.objbrowser',
 'prettyqt.positioning',
 'prettyqt.qml',
 'prettyqt.qt',
 'prettyqt.qt.QtBluetooth',
 'prettyqt.qt.QtCharts',
 'prettyqt.qt.QtCore',
 'prettyqt.qt.QtGui',
 'prettyqt.qt.QtMultimedia',
 'prettyqt.qt.QtMultimediaWidgets',
 'prettyqt.qt.QtNetwork',
 'prettyqt.qt.QtPositioning',
 'prettyqt.qt.QtQml',
 'prettyqt.qt.QtQuick',
 'prettyqt.qt.QtSvg',
 'prettyqt.qt.QtTest',
 'prettyqt.qt.QtTextToSpeech',
 'prettyqt.qt.QtWebEngineCore',
 'prettyqt.qt.QtWebEngineWidgets',
 'prettyqt.qt.QtWidgets',
 'prettyqt.qt.QtWinExtras',
 'prettyqt.qthelp',
 'prettyqt.quick',
 'prettyqt.scintilla',
 'prettyqt.svg',
 'prettyqt.syntaxhighlighters',
 'prettyqt.syntaxhighlighters.pygments',
 'prettyqt.texttospeech',
 'prettyqt.utils',
 'prettyqt.webenginecore',
 'prettyqt.webenginewidgets',
 'prettyqt.widgets',
 'prettyqt.winextras']

package_data = \
{'': ['*'], 'prettyqt': ['localization/*']}

install_requires = \
['bidict>=0,<1',
 'deprecated>=1.2.10,<2.0.0',
 'docutils>=0,<1',
 'orjson>=3.2.0,<4.0.0',
 'pygments>=2.6.1,<3.0.0',
 'qstylizer>=0.1.9,<0.2.0',
 'qtawesome>=1,<2',
 'qtpy>=1.9.0,<2.0.0',
 'regex>=2020.6.8,<2021.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0,<1'],
 ':sys_platform == "darwin"': ['darkdetect>=0,<1'],
 'pyqt5': ['PyQt5>=5.15.0,<6.0.0',
           'PyQtWebEngine>=5.15.0,<6.0.0',
           'PyQtChart>=5.15.0,<6.0.0',
           'QScintilla>=2.11.5,<3.0.0'],
 'pyside2:python_version >= "3.7" and python_version < "3.9"': ['pyside2>=5.15.0,<6.0.0']}

entry_points = \
{'console_scripts': ['regexeditor = '
                     'prettyqt.custom_widgets.regexeditor.editor:run']}

setup_kwargs = {
    'name': 'prettyqt',
    'version': '0.140.3',
    'description': 'A pythonic layer on top of qtpy',
    'long_description': '# prettyqt: Pythonic layer on top of PyQt5 / PySide2\n[![PyPI Latest Release](https://img.shields.io/pypi/v/prettyqt.svg)](https://pypi.org/project/prettyqt/)\n[![Package Status](https://img.shields.io/pypi/status/prettyqt.svg)](https://pypi.org/project/prettyqt/)\n[![License](https://img.shields.io/pypi/l/prettyqt.svg)](https://github.com/prettyqt-dev/prettyqt/blob/master/LICENSE)\n[![Travis Build Status](https://travis-ci.org/prettyqt-dev/prettyqt.svg?branch=master)](https://travis-ci.org/prettyqt-dev/prettyqt)\n[![CodeCov](https://codecov.io/gh/phil65/PrettyQt/branch/master/graph/badge.svg)](https://codecov.io/gh/phil65/PrettyQt)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![PyUp](https://pyup.io/repos/github/phil65/PrettyQt/shield.svg)](https://pyup.io/repos/github/phil65/PrettyQt/)\n\n## What is it?\n\n**PrettyQt** is a Python package that provides a pythonic layer on top of the GUI frameworks PyQt5 / PySide2.\n\n## Main Features\nHere are just a few of the things that PrettyQt does well:\n\n  - Large parts of the Qt API are available in a **PEP-8**-compliant way.\n  - Several predefined widgets, validators, models, syntax highlighters are included.\n  - A regex module based on QRegularExpression with the same API as Pythons core re module.\n\n\n   [widgets]: https://phil65.github.io/PrettyQt/widgets.html\n   [validators]: https://phil65.github.io/PrettyQt/validators.html\n   [syntaxhighlighters]: https://phil65.github.io/PrettyQt/syntaxhighlighters.html\n   [models]: https://phil65.github.io/PrettyQt/models.html\n\n\n## Where to get it\nThe source code is currently hosted on GitHub at:\nhttps://github.com/phil65/PrettyQt\n\nThe latest released version are available at the [Python\npackage index](https://pypi.org/project/prettyqt).\n\n```sh\n# or PyPI\npip install prettyqt\n```\n\n## Dependencies\n- [qtpy](https://pypi.org/project/prettyqt)\n- [qtawesome](https://pypi.org/project/qtawesome)\n- [bidict](https://pypi.org/project/bidict)\n- [orjson](https://pypi.org/project/orjson)\n- [regex](https://pypi.org/project/regex)\n- [docutils](https://pypi.org/project/docutils)\n\n\n## Installation from sources\n\nThis project uses poetry for dependency management and packaging. Install this first.\nIn the `prettyqt` directory (same one where you found this file after\ncloning the git repo), execute:\n\n```sh\npoetry install\n```\n\n## License\n[MIT](LICENSE)\n\n## Documentation\nThe official documentation is hosted on Github Pages: https://phil65.github.io/PrettyQt/\n\n## Contributing to pandas [![Open Source Helpers](https://www.codetriage.com/phil65/prettyqt/badges/users.svg)](https://www.codetriage.com/phil65/prettyqt)\n\nAll contributions, bug reports, bug fixes, documentation improvements, enhancements, and ideas are welcome.\n\nOr maybe through using PrettyQt you have an idea of your own or are looking for something in the documentation and thinking ‘this can be improved’...you can do something about it!\n',
    'author': 'phil65',
    'author_email': 'philipptemminghoff@googlemail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/phil65/prettyqt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Optional, List
import logging

from qtpy import QtWidgets

from prettyqt import core, constants
from prettyqt.utils import columnitem

logger = logging.getLogger(__name__)


class ColumnItemModel(core.AbstractItemModel):
    """Model that provides an interface to an objectree that is build of TreeItems."""

    def __init__(
        self,
        attr_cols: Optional[List[columnitem.ColumnItem]] = None,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent)
        self._attr_cols = attr_cols

    def columnCount(self, _parent=None):
        """Return the number of columns in the tree."""
        return len(self._attr_cols)

    def data(self, index, role):
        """Return the tree item at the given index and role."""
        if not index.isValid():
            return None

        col = index.column()
        tree_item = index.internalPointer()

        if role in [constants.DISPLAY_ROLE, constants.EDIT_ROLE]:
            func = self._attr_cols[col].label
            if func is None:
                return ""
            attr = func(tree_item)
            return attr.replace("\n", " ")
        elif role == constants.DECORATION_ROLE:
            return self._attr_cols[col].get_decoration(tree_item)
        elif role == constants.CHECKSTATE_ROLE:
            return self._attr_cols[col].get_checkstate(tree_item)
        elif role == constants.ALIGNMENT_ROLE:
            return self._attr_cols[col].get_alignment(tree_item)
        elif role == constants.FOREGROUND_ROLE:
            return self._attr_cols[col].get_foreground_color(tree_item)
        elif role == constants.BACKGROUND_ROLE:
            return self._attr_cols[col].get_background_color(tree_item)
        elif role == constants.FONT_ROLE:
            return self._attr_cols[col].get_font(tree_item)
        else:
            return None

    def flags(self, index):
        if not index.isValid():
            return constants.NO_CHILDREN
        col = index.column()
        return self._attr_cols[col].get_flag()

    def headerData(self, section, orientation, role):
        if orientation == constants.HORIZONTAL and role == constants.DISPLAY_ROLE:
            return self._attr_cols[section].name
        else:
            return None



# -*- coding: utf-8 -*-

from deprecated import deprecated
from qtpy import QtCore, QtWidgets, QtGui
from qtpy.QtCharts import QtCharts

from prettyqt import charts, core, gui, widgets
from prettyqt.utils import bidict, InvalidParamError

RUBBER_BANDS = bidict(
    none=QtCharts.QChartView.NoRubberBand,
    vertical=QtCharts.QChartView.VerticalRubberBand,
    horizontal=QtCharts.QChartView.HorizontalRubberBand,
    rectangle=QtCharts.QChartView.RectangleRubberBand,
)

ALIGNMENTS = bidict(
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
)

ZOOM_IN_FACTOR = 1.1
ZOOM_OUT_FACTOR = 1.0 / ZOOM_IN_FACTOR
SCROLL_STEP_SIZE = 10


class ChartView(QtCharts.QChartView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        chart = charts.Chart()
        self.setChart(chart)
        self.setRenderHint(gui.Painter.Antialiasing)
        self.set_rubber_band("rectangle")
        # self.setDragMode(self.RubberBandDrag)

    def keyPressEvent(self, event: QtGui.QKeyEvent):
        """Handle keypress events to allow navigation via keyboard."""
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            self.chart().zoomReset()
        elif key == QtCore.Qt.Key_Plus:
            self.chart().zoom_by_factor(ZOOM_IN_FACTOR)
        elif key == QtCore.Qt.Key_Minus:
            self.chart().zoom_by_factor(ZOOM_OUT_FACTOR)
        elif key == QtCore.Qt.Key_Left:
            self.chart().scroll(-SCROLL_STEP_SIZE, 0)
        elif key == QtCore.Qt.Key_Right:
            self.chart().scroll(SCROLL_STEP_SIZE, 0)
        elif key == QtCore.Qt.Key_Up:
            self.chart().scroll(0, SCROLL_STEP_SIZE)
        elif key == QtCore.Qt.Key_Down:
            self.chart().scroll(0, -SCROLL_STEP_SIZE)
        elif key == QtCore.Qt.Key_0:
            self.chart().apply_nice_numbers()
        else:
            return super().keyPressEvent(event)
        event.accept()

    def wheelEvent(self, event: QtGui.QWheelEvent):
        """Handle wheel event for zooming."""
        fct = ZOOM_IN_FACTOR if event.angleDelta().y() > 0 else ZOOM_OUT_FACTOR
        self.chart().zoom_by_factor(fct)

    def mouseReleaseEvent(self, event: QtGui.QMouseEvent):
        """Override to allow dragging the chart."""
        if event.button() == QtCore.Qt.RightButton:
            widgets.Application.restoreOverrideCursor()
            event.accept()
            return None
        super().mouseReleaseEvent(event)

    def mousePressEvent(self, event: QtGui.QMouseEvent):
        """Override to allow dragging the chart."""
        if event.button() == QtCore.Qt.RightButton:
            cursor = gui.Cursor(QtCore.Qt.SizeAllCursor)
            widgets.Application.setOverrideCursor(cursor)
            self.last_mouse_pos = event.pos()
            event.accept()

        super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QtGui.QMouseEvent):
        """Override to allow dragging the chart."""
        # pan the chart with a middle mouse drag
        if event.buttons() & QtCore.Qt.RightButton:
            if not self.last_mouse_pos:
                return None
            pos_diff = event.pos() - self.last_mouse_pos
            self.chart().scroll(-pos_diff.x(), pos_diff.y())

            self.last_mouse_pos = event.pos()
            event.accept()

            widgets.Application.restoreOverrideCursor()

        super().mouseMoveEvent(event)

    @deprecated(reason="This method is deprecated, use save_as_image instead.")
    def save(self):
        self.save_as_image()

    @core.Slot()
    def save_as_image(self):
        """Let user choose folder and save chart as an image file."""
        dlg = widgets.FileDialog(mode="save", caption="Save image")
        filters = {"Bmp files": [".bmp"], "Jpeg files": [".jpg"], "Png files": [".png"]}
        dlg.set_extension_filter(filters)
        filename = dlg.open_file()
        if not filename:
            return None
        self.chart().show_legend()
        image = self.get_image()
        image.save(filename[0])
        self.chart().hide_legend()

    def get_image(self) -> QtGui.QPixmap:
        image = self.grab()
        gl_widget = self.findChild(QtWidgets.QOpenGLWidget)
        if gl_widget:
            d = gl_widget.mapToGlobal(core.Point()) - self.mapToGlobal(core.Point())
            with gui.Painter(image) as painter:
                painter.set_composition_mode("source_atop")
                painter.drawImage(d, gl_widget.grabFramebuffer())
        return image

    def set_rubber_band(self, typ: str):
        """Set the rubber band type.

        Allowed values are "none", "horizontal", "vertical", "rectangle"

        Args:
            typ: rubber band type

        Raises:
            InvalidParamError: rubber band type does not exist
        """
        if typ not in RUBBER_BANDS:
            raise InvalidParamError(typ, RUBBER_BANDS)
        self.setRubberBand(RUBBER_BANDS[typ])

    def get_rubber_band(self) -> str:
        """Return current rubber band type.

        Possible values are "none", "horizontal", "vertical", "rectangle"

        Returns:
            Rubber band type
        """
        return RUBBER_BANDS.inv[self.rubberBand()]

    # def hide_legend(self):
    #     self.chart().hide_legend()

    # def show_legend(self):
    #     self.chart().show_legend()

    # def set_legend_alignment(self, alignment: str):
    #     if alignment not in ALIGNMENTS:
    #         raise ValueError(f"{alignment!r} not a valid alignment.")
    #     self.chart().legend().setAlignment(ALIGNMENTS[alignment])



from typing import Optional

from qtpy import QtMultimedia, QtCore

from prettyqt import constants, core


class PlaylistModel(core.AbstractItemModel):

    HEADER = ["Name"]

    def __init__(self, parent: Optional[QtCore.QObject] = None):
        super().__init__(parent)
        self._playlist = None

    def rowCount(self, parent=core.ModelIndex()):
        return (
            self._playlist.mediaCount()
            if self._playlist is not None and not parent.isValid()
            else 0
        )

    def columnCount(self, parent=core.ModelIndex()):
        return len(self.HEADER) if not parent.isValid() else 0

    def index(self, row, column, parent=core.ModelIndex()):
        return (
            self.createIndex(row, column)
            if self._playlist is not None
            and not parent.isValid()
            and 0 <= row < self._playlist.mediaCount()
            and 0 <= column < len(self.HEADER)
            else core.ModelIndex()
        )

    def parent(self, child):
        return core.ModelIndex()

    def data(self, index, role=constants.DISPLAY_ROLE):
        if index.isValid() and role == constants.DISPLAY_ROLE:
            if index.column() == 0:
                location = self._playlist.media(index.row()).canonicalUrl()
                return core.FileInfo(location.path()).fileName()

            return self.m_data[index]

        return None

    def playlist(self) -> QtMultimedia.QMediaPlaylist:
        return self._playlist

    def set_playlist(self, playlist: QtMultimedia.QMediaPlaylist):
        if self._playlist is not None:
            self._playlist.mediaAboutToBeInserted.disconnect(self.beginInsertItems)
            self._playlist.mediaInserted.disconnect(self.endInsertRows)
            self._playlist.mediaAboutToBeRemoved.disconnect(self.beginRemoveItems)
            self._playlist.mediaRemoved.disconnect(self.endRemoveRows)
            self._playlist.mediaChanged.disconnect(self.change_items)

        with self.reset_model():
            self._playlist = playlist

            if self._playlist is not None:
                self._playlist.mediaAboutToBeInserted.connect(
                    lambda x, y: self.beginInsertRows(core.ModelIndex(), x, y)
                )
                self._playlist.mediaInserted.connect(self.endInsertRows)
                self._playlist.mediaAboutToBeRemoved.connect(
                    lambda x, y: self.beginRemoveRows(core.ModelIndex(), x, y)
                )
                self._playlist.mediaRemoved.connect(self.endRemoveRows)
                self._playlist.mediaChanged.connect(self.change_items)

    def change_items(self, start: int, end: int):
        self.dataChanged.emit(self.index(start, 0), self.index(end, len(self.HEADER)))



# -*- coding: utf-8 -*-

from typing import Union, Optional

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict

SHAPE = bidict(line=QtWidgets.QRubberBand.Line, rectangle=QtWidgets.QRubberBand.Rectangle)


QtWidgets.QRubberBand.__bases__ = (widgets.Widget,)


class RubberBand(QtWidgets.QRubberBand):
    def __init__(
        self, shape: Union[str, int], parent: Optional[QtWidgets.QWidget] = None
    ):
        if isinstance(shape, str):
            shape = SHAPE[shape]
        super().__init__(shape, parent)

    def get_shape(self) -> str:
        return SHAPE.inv[self.shape()]



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QStyleOptionComplex.__bases__ = (widgets.StyleOption,)


class StyleOptionComplex(QtWidgets.QStyleOptionComplex):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtMultimedia


class MediaTimeInterval(QtMultimedia.QMediaTimeInterval):
    def __repr__(self):
        return f"MediaTimeInterval({self.start()}, {self.end()})"

    def __contains__(self, val: int):
        return self.contains(val)

    def __reduce__(self):
        return (self.__class__, (self.start(), self.end()))


if __name__ == "__main__":
    interval = MediaTimeInterval(0, 1000)
    print(repr(interval))
    assert 500 in interval
    import pickle

    with open("data.pkl", "wb") as jar:
        pickle.dump(interval, jar)
    with open("data.pkl", "rb") as jar:
        w = pickle.load(jar)



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import gui, core
from prettyqt.utils import bidict, InvalidParamError


STATES = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)

CHILD_INDICATOR_POLICY = bidict(
    show=QtWidgets.QTreeWidgetItem.ShowIndicator,
    dont_show=QtWidgets.QTreeWidgetItem.DontShowIndicator,
    dont_show_when_childless=QtWidgets.QTreeWidgetItem.DontShowIndicatorWhenChildless,
)


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
    def __repr__(self):
        return "TreeWidgetItem()"

    def serialize_fields(self):
        icon = self.icon(0)
        return dict(
            text=[self.text(i) for i in range(self.columnCount())],
            tool_tip=[self.toolTip(i) for i in range(self.columnCount())],
            status_tip=[self.statusTip(i) for i in range(self.columnCount())],
            checkstate=[self.get_checkstate(i) for i in range(self.columnCount())],
            icon=gui.Icon(icon) if not icon.isNull() else None,
            data=[self.data(i, QtCore.Qt.UserRole) for i in range(self.columnCount())],
        )

    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def set_icon(self, icon: gui.icon.IconType, column: int = 0):
        """Set the icon for the action.

        Args:
            icon: icon to use
            column: column
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(column, icon)

    def get_background(self, column: int = 0) -> gui.Brush:
        return gui.Brush(self.background(column))

    def get_foreground(self, column: int = 0) -> gui.Brush:
        return gui.Brush(self.foreground(column))

    def get_font(self, column: int = 0) -> gui.Font:
        return gui.Font(self.font(column))

    def get_icon(self, column: int = 0) -> gui.Icon:
        return gui.Icon(self.icon(column))

    def set_checkstate(self, state: str, column: int = 0):
        """Set checkstate of the checkbox.

        valid values are: unchecked, partial, checked

        Args:
            state: checkstate to use
            column: column

        Raises:
            InvalidParamError: invalid checkstate
        """
        if state not in STATES:
            raise InvalidParamError(state, STATES)
        self.setCheckState(column, STATES[state])

    def get_checkstate(self, column: int = 0) -> str:
        """Return checkstate.

        possible values are "unchecked", "partial", "checked"

        Args:
            column: column

        Returns:
            checkstate
        """
        return STATES.inv[self.checkState(column)]

    def set_child_indicator_policy(self, policy: str):
        """Set the child indicator policy.

        Allowed values are "show", "dont_show", "dont_show_when_childless"

        Args:
            policy: child indicator policy

        Raises:
            InvalidParamError: policy does not exist
        """
        if policy not in CHILD_INDICATOR_POLICY:
            raise InvalidParamError(policy, CHILD_INDICATOR_POLICY)
        self.setChildIndicatorPolicy(CHILD_INDICATOR_POLICY[policy])

    def get_child_indicator_policy(self) -> str:
        """Return current child indicator policy.

        Possible values: "show", "dont_show", "dont_show_when_childless"

        Returns:
            child indicator policy
        """
        return CHILD_INDICATOR_POLICY.inv[self.childIndicatorPolicy()]


if __name__ == "__main__":
    item = TreeWidgetItem()
    item.setData(0, 1000, "test")



# -*- coding: utf-8 -*-

from qtpy import QtMultimedia

from prettyqt import multimedia


QtMultimedia.QAudioRecorder.__bases__ = (multimedia.MediaRecorder,)


class AudioRecorder(QtMultimedia.QAudioRecorder):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QAbstractEventDispatcher.__bases__ = (core.Object,)


class AbstractEventDispatcher(QtCore.QAbstractEventDispatcher):
    pass



# -*- coding: utf-8 -*-

import pathlib
from typing import Union

from qtpy import QtMultimedia

from prettyqt import core, multimedia
from prettyqt.utils import bidict


STATES = bidict(
    stopped=QtMultimedia.QMediaRecorder.StoppedState,
    recording=QtMultimedia.QMediaRecorder.RecordingState,
    paused=QtMultimedia.QMediaRecorder.PausedState,
)

ERRORS = bidict(
    none=QtMultimedia.QMediaRecorder.NoError,
    resource=QtMultimedia.QMediaRecorder.ResourceError,
    format=QtMultimedia.QMediaRecorder.FormatError,
    out_of_space=QtMultimedia.QMediaRecorder.OutOfSpaceError,
)

MEDIA_STATUS = bidict(
    unavailable=QtMultimedia.QMediaRecorder.UnavailableStatus,
    unloaded=QtMultimedia.QMediaRecorder.UnloadedStatus,
    loading=QtMultimedia.QMediaRecorder.LoadingStatus,
    loaded=QtMultimedia.QMediaRecorder.LoadedStatus,
    starting=QtMultimedia.QMediaRecorder.StartingStatus,
    recording=QtMultimedia.QMediaRecorder.RecordingStatus,
    paused=QtMultimedia.QMediaRecorder.PausedStatus,
    finalizing=QtMultimedia.QMediaRecorder.FinalizingStatus,
)

AVAILABILITY_STATUS = bidict(
    available=QtMultimedia.QMultimedia.Available,
    service_missing=QtMultimedia.QMultimedia.ServiceMissing,
    resource_error=QtMultimedia.QMultimedia.ResourceError,
    busy=QtMultimedia.QMultimedia.Busy,
)


QtMultimedia.QMediaRecorder.__bases__ = (core.Object, multimedia.MediaBindableInterface)


class MediaRecorder(QtMultimedia.QMediaRecorder):
    def set_video_settings(self, settings):
        if isinstance(settings, multimedia.videoencodersettings.Settings):
            settings = settings.VideoEncoderSettings.from_dataclass(settings)
        elif isinstance(settings, dict):
            settings = settings.VideoEncoderSettings.from_dict(settings)
        self.setVideoSettings(settings)

    def get_video_settings(self) -> multimedia.VideoEncoderSettings:
        return multimedia.VideoEncoderSettings(self.videoSettings())

    def set_audio_settings(self, settings):
        if isinstance(settings, multimedia.audioencodersettings.Settings):
            settings = settings.AudioEncoderSettings.from_dataclass(settings)
        elif isinstance(settings, dict):
            settings = settings.AudioEncoderSettings.from_dict(settings)
        self.setAudioSettings(settings)

    def get_audio_settings(self) -> multimedia.AudioEncoderSettings:
        return multimedia.AudioEncoderSettings(self.audioSettings())

    def get_availability(self) -> str:
        """Return availability status.

        Possible values: "available", "service_missing", "resource_error", "busy"

        Returns:
            availability status
        """
        return AVAILABILITY_STATUS.inv[self.availability()]

    def set_output_location(self, path: Union[pathlib.Path, str]):
        self.setOutputLocation(core.Url(path))

    def get_output_location(self) -> str:
        return str(core.Url(self.outputLocation()))



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import core

QtWidgets.QGraphicsTransform.__bases__ = (core.Object,)


class GraphicsTransform(QtWidgets.QGraphicsTransform):
    pass


if __name__ == "__main__":
    transform = GraphicsTransform()



# -*- coding: utf-8 -*-

from qtpy import QtGui, QtCore

from prettyqt import core
from prettyqt.utils import colors
from prettyqt.utils import bidict, InvalidParamError

PEN_STYLES = bidict(
    none=QtCore.Qt.NoPen,
    solid=QtCore.Qt.SolidLine,
    dash=QtCore.Qt.DashLine,
    dot=QtCore.Qt.DotLine,
    dash_dot=QtCore.Qt.DashDotLine,
    dash_dot_dot=QtCore.Qt.DashDotDotLine,
    custom_dash=QtCore.Qt.CustomDashLine,
)

CAP_STYLES = bidict(
    flat=QtCore.Qt.FlatCap, square=QtCore.Qt.SquareCap, round=QtCore.Qt.RoundCap
)

JOIN_STYLES = bidict(
    miter=QtCore.Qt.MiterJoin,
    bevel=QtCore.Qt.BevelJoin,
    round=QtCore.Qt.RoundJoin,
    svg_miter=QtCore.Qt.SvgMiterJoin,
)


class Pen(QtGui.QPen):
    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def set_color(self, color: colors.ColorType):
        color = colors.get_color(color)
        self.setColor(color)

    def set_cap_style(self, style: str):
        """Set cap style to use.

        Allowed values are "flat", "square", "round"

        Args:
            style: cap style to use

        Raises:
            InvalidParamError: cap style does not exist
        """
        if style not in CAP_STYLES:
            raise InvalidParamError(style, CAP_STYLES)
        self.setCapStyle(CAP_STYLES[style])

    def get_cap_style(self) -> str:
        """Return current cap style.

        Possible values: "flat", "square", "round"

        Returns:
            cap style
        """
        return CAP_STYLES.inv[self.capStyle()]

    def set_join_style(self, style: str):
        """Set join style to use.

        Allowed values are "miter", "bevel", "round", "svg_miter"

        Args:
            style: join style to use

        Raises:
            InvalidParamError: join style does not exist
        """
        if style not in JOIN_STYLES:
            raise InvalidParamError(style, JOIN_STYLES)
        self.setJoinStyle(JOIN_STYLES[style])

    def get_join_style(self) -> str:
        """Return current join style.

        Possible values: "miter", "bevel", "round", "svg_miter"

        Returns:
            join style
        """
        return JOIN_STYLES.inv[self.joinStyle()]

    def set_style(self, style: str):
        """Set pen style to use.

        Allowed values are "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot",
        "custom_dash"

        Args:
            style: pen style to use

        Raises:
            InvalidParamError: pen style does not exist
        """
        if style not in PEN_STYLES:
            raise InvalidParamError(style, PEN_STYLES)
        self.setStyle(PEN_STYLES[style])

    def get_style(self) -> str:
        """Return current pen style.

        Possible values: "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot",
        "custom_dash"

        Returns:
            pen style
        """
        return PEN_STYLES.inv[self.style()]



# -*- coding: utf-8 -*-

from typing import Union, Optional
import pathlib

from qtpy import QtCore, QtWidgets, QtGui

from prettyqt import gui, widgets
from prettyqt.utils import bidict


H_ALIGNMENTS = bidict(
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    center=QtCore.Qt.AlignHCenter,
    justify=QtCore.Qt.AlignJustify,
)

V_ALIGNMENTS = bidict(
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
    center=QtCore.Qt.AlignVCenter,
    baseline=QtCore.Qt.AlignBaseline,
)


QtWidgets.QSplashScreen.__bases__ = (widgets.Widget,)


class SplashScreen(QtWidgets.QSplashScreen):
    def __init__(
        self, path: Union[pathlib.Path, str, QtGui.QPixmap], width: Optional[int] = None
    ):
        pix = gui.Pixmap(str(path)) if not isinstance(path, QtGui.QPixmap) else path
        if width:
            pix = pix.scaledToWidth(width)
        super().__init__(pix)
        self.setWindowFlags(
            QtCore.Qt.WindowStaysOnTopHint | QtCore.Qt.FramelessWindowHint
        )
        self.setEnabled(False)

    def __enter__(self):
        self.show()
        return self

    def __exit__(self, typ, value, traceback):
        self.hide()

    def set_text(self, text, color="black", h_align="center", v_align="bottom"):
        self.showMessage(
            text,
            color=gui.Color(color),
            alignment=H_ALIGNMENTS[h_align] | V_ALIGNMENTS[v_align],
        )



# -*- coding: utf-8 -*-

from typing import Union, Optional

from qtpy import QtCore, QtWidgets

from prettyqt import core, widgets
from prettyqt.utils import bidict


ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical)


QtWidgets.QScrollBar.__bases__ = (widgets.AbstractSlider,)


class ScrollBar(QtWidgets.QScrollBar):

    value_changed = core.Signal(int)

    def __init__(
        self,
        orientation: Union[int, str] = "horizontal",
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        if orientation in ORIENTATIONS:
            orientation = ORIENTATIONS[orientation]
        super().__init__(orientation, parent)
        self.valueChanged.connect(self.on_value_change)



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict


NAME_TYPES = bidict(
    default=QtCore.QTimeZone.DefaultName,
    long=QtCore.QTimeZone.LongName,
    short=QtCore.QTimeZone.ShortName,
    offset=QtCore.QTimeZone.OffsetName,
)

TIME_TYPES = bidict(
    standard=QtCore.QTimeZone.StandardTime,
    daylight=QtCore.QTimeZone.DaylightTime,
    generic=QtCore.QTimeZone.GenericTime,
)


class TimeZone(QtCore.QTimeZone):
    def __init__(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], str):
            super().__init__(args[0].encode())
        else:
            super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"TimeZone({self.get_id()!r})"

    def __str__(self):
        return self.get_id()

    def __reduce__(self):
        return (self.__class__, (self.get_id(),))

    def get_id(self) -> str:
        return bytes(self.id()).decode()

    # def get_value(self) -> datetime.datetime:
    #     try:
    #         return self.toPython()
    #     except TypeError:
    #         return self.toPyTimeZone()


if __name__ == "__main__":
    date = core.TimeZone(2000, 11, 11)
    dt = TimeZone(date)



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui


QtGui.QLinearGradient.__bases__ = (gui.Gradient,)


class LinearGradient(QtGui.QLinearGradient):
    def serialize_fields(self):
        start = self.start()
        final_stop = self.finalStop()
        return dict(start=(start[0], start[1]), final_stop=(final_stop[0], final_stop[1]))
